@@ -10,17 +10,32 @@ def wget(url, verify=True):
 		open(to,'wb').write(resp.content)
 	return to
 
-def extract_ova_from_zip(local_zipfile):
+def extract_file_from_zip(local_zipfile, extractedfile):
 	import zipfile
 
-	ovafile = os.path.basename(local_zipfile).replace('.zip','.ova')
-	if not os.path.exists(ovafile):
+	if not os.path.exists(extractedfile):
 		cur_folder = os.path.abspath(os.curdir)
 		with zipfile.ZipFile(local_zipfile,"r") as zip_ref:
 			zip_ref.extractall(cur_folder)
 		os.remove(local_zipfile)
 
-	return ovafile if os.path.exists(ovafile) else None
+	return extractedfile if os.path.exists(extractedfile) else None
+
+
+def extract_ova_from_zip(local_zipfile):
+	if False:
+		import zipfile
+
+		ovafile = os.path.basename(local_zipfile).replace('.zip','.ova')
+		if not os.path.exists(ovafile):
+			cur_folder = os.path.abspath(os.curdir)
+			with zipfile.ZipFile(local_zipfile,"r") as zip_ref:
+				zip_ref.extractall(cur_folder)
+			os.remove(local_zipfile)
+
+		return ovafile if os.path.exists(ovafile) else None
+	else:
+		return extract_file_from_zip(local_zipfile, os.path.basename(local_zipfile).replace('.zip','.ova'))
 
 def open_port():
 	"""
@@ -119,6 +134,14 @@ class dock:
 		if self.nonet:
 			network = "--network none" #https://docs.docker.com/network/none/
 
+		my_save_host_dir = ''
+		if self.save_host_dir:
+			if 'HOSTDIR' in os.environ:
+				past_dir,current_dir = os.environ['HOSTDIR'], os.path.abspath(os.curdir).replace('/sync/','')
+				my_save_host_dir = '--env="HOSTDIR={0}/{1}"'.format(past_dir,current_dir)
+			else:
+				my_save_host_dir = '--env="HOSTDIR={0}"'.format(dir)
+
 		return str(self.clean()+";" if self.preClean else "") + "{0} run ".format(self.docker) + " ".join([
 			dockerInDocker,
 			'--rm' if self.remove else '',
@@ -129,7 +152,7 @@ class dock:
 			getPort(self.ports),
 			'--mac-address ' + str(self.macaddress) if self.macaddress else '',
 			self.extra if self.extra else '',
-			'--env="HOSTDIR={0}"'.format(dir) if self.save_host_dir else '',
+			my_save_host_dir,
 			self.image,
 			cmd
 		]) + str(self.clean()+";" if self.postClean else "")
@@ -205,7 +228,7 @@ class vb:
 		if self.sharedfolder:
 			self.__shared_folder(self.sharedfolder)
 		
-		for file in self.uploadfiles:
+		for file in list(self.uploadfiles):
 			self.uploadfile(file)
 
 		if False:			
@@ -249,6 +272,9 @@ class vb:
 				os.remove(self.ovafile)
 
 		exe(cmd)
+	
+	def destroy(self, deletefiles:bool=True):
+		self.clean(deletefiles)
 
 @dataclass
 class vagrant(object):
@@ -282,6 +308,9 @@ class vagrant(object):
 
 		if self.save_files is None or type(self.save_files) is tuple:
 			self.save_files = []
+		
+		if self.vmdate is None or type(self.vmdate) is tuple:
+			self.vmdate = None
 
 	@property
 	def vagrant_name(self):
@@ -329,7 +358,8 @@ class vagrant(object):
 		return """ win10.vm.provision "file", source: "{0}", destination: "{1}\\\\{0}" """.format(foil, directory)
 
 
-	def prep(self):		
+	def prep(self):
+		self.uploadfiles = list(self.uploadfiles)	
 		self.uploadfiles += [self.write_startup_file()]
 		uploading_file_strings = []
 		for foil in self.uploadfiles:
@@ -429,12 +459,15 @@ end
 		self.vagrant_name
 		exe("{0} controlvm {1} poweroff".format(self.vb_box_exe, self.vagrant_name))
 	
-	def destroy(self):
+	def destroy(self,emptyflag=False):
 		self.vagrant_name
 		exe(""" vagrant destroy -f """)
 		for foil in ["Vagrant", "on_start*", "on_login*"]:
 			exe("rm {0}".format(foil))
 		exe("yes|rm -r .vagrant/")
-		for foil in self.uploadfiles:
+		for foil in list(self.uploadfiles):
 			if foil not in self.save_files:
 				exe("rm {0}".format(foil))
+
+	def clean(self,emptyflag=False):
+		self.destroy(emptyflag)
